# -*- coding: utf-8 -*-
#
# Copyright (C) 2013-2014  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from cnfft3 cimport nfft_plan

cdef object nfft_flags_dict
cdef object fftw_flags_dict
cdef object nfft_supported_flags_tuple

cdef class NFFT(object):
    cdef nfft_plan _plan
    cdef int _d
    cdef int _M
    cdef int _m
    cdef object _N
    cdef object _n
    cdef object _dtype
    cdef object _flags
    cdef object _f   
    cdef object _f_hat
    cdef object _x
    cdef void _precompute(self)
    cdef void _trafo(self)
    cdef void _trafo_direct(self)    
    cdef void _adjoint(self)
    cdef void _adjoint_direct(self)




# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from cnfft3 cimport fftw_complex

cdef extern from "nfft3util.h":

    void nfft_vrand_unit_complex (fftw_complex *x, int n)
 	    # Inits a vector of random complex numbers in \
        # $[0,1]\times[0,1]{\rm i}$ .

    void nfft_vrand_shifted_unit_double (double *x, int n)
        # Inits a vector of random double numbers in $[-1/2,1/2]$ .

    void nfft_voronoi_weights_1d (double *w, double *x, int M)
 	    # Computes non periodic voronoi weights, \
        # assumes ordered nodes $x_j$.

    void nfft_voronoi_weights_S2(double *w, double *xi, int M)
        # Computes voronoi weights for nodes on the sphere S^2. */




# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '1.3.2'
version = '1.3.2'
full_version = '1.3.2'
git_revision = 'c45b7027a79c9633e8ffa84532f4bfa092e510a9'
release = True

if not release:
    version = full_version



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from __future__ import division
import numpy
from numpy import pi
from numpy.testing import assert_allclose
from pynfft.nfft import NFFT
from pynfft.util import vrand_unit_complex, vrand_shifted_unit_double


def fdft(x, f_hat):
    N = f_hat.shape
    d = x.shape[-1]
    k = numpy.mgrid[[slice(-Nt/2, Nt/2) for Nt in N]]
    k = k.reshape([d, -1])
    x = x.reshape([-1, d])
    F = numpy.exp(-2j * pi * numpy.dot(x, k))
    f_dft = numpy.dot(F, f_hat.ravel())
    return f_dft

def rdft(x, f, N):
    d = x.shape[-1]
    k = numpy.mgrid[[slice(-Nt/2, Nt/2) for Nt in N]]
    k = k.reshape([d, -1])
    x = x.reshape([-1, d])
    F = numpy.exp(-2j * pi * numpy.dot(x, k))
    f_hat_dft = numpy.dot(numpy.conjugate(F).T, f)
    f_hat = f_hat_dft.reshape(N)        
    return f_hat

def check_forward_nfft(plan):
    vrand_unit_complex(plan.f_hat.ravel())
    assert_allclose(plan.trafo(), fdft(plan.x, plan.f_hat))

def check_forward_ndft(plan):
    vrand_unit_complex(plan.f_hat.ravel())
    assert_allclose(plan.trafo(use_dft=True), fdft(plan.x, plan.f_hat))

def check_adjoint_nfft(plan):
    vrand_unit_complex(plan.f.ravel())
    assert_allclose(plan.adjoint(), rdft(plan.x, plan.f, plan.N))

def check_adjoint_ndft(plan):
    vrand_unit_complex(plan.f.ravel())
    assert_allclose(plan.adjoint(use_dft=True), rdft(plan.x, plan.f, plan.N))

tested_nfft_args = (
    (8, 8, dict(m=6)),
    (16, 16, dict()),
    (24, 24, dict()),
    (32, 32, dict()),
    (64, 64, dict()),
    ((8, 8), 8*8, dict(m=6)),
    ((16, 16), 16*16, dict()),
    ((24, 24), 24*24, dict()),
    ((32, 32), 32*32, dict()),
    ((64, 64), 64*64, dict()),
    ((8, 8, 8), 8*8*8, dict(m=6)),
    ((16, 16, 8), 8*8*8, dict(m=6)),
    ((16, 16, 16), 16*16*16, dict()),
)

def test_forward_nfft():
    for N, M, nfft_kwargs in tested_nfft_args:
        plan = NFFT(N, M, **nfft_kwargs)
        vrand_shifted_unit_double(plan.x.ravel())
        plan.precompute()
        yield check_forward_nfft, plan

def test_forward_ndft():
    for N, M, nfft_kwargs in tested_nfft_args:
        plan = NFFT(N, M, **nfft_kwargs)
        vrand_shifted_unit_double(plan.x.ravel())
        plan.precompute()
        yield check_forward_ndft, plan

def test_adjoint_nfft():
    for N, M, nfft_kwargs in tested_nfft_args:
        plan = NFFT(N, M, **nfft_kwargs)
        vrand_shifted_unit_double(plan.x.ravel())
        plan.precompute()
        yield check_adjoint_nfft, plan

def test_adjoint_ndft():
    for N, M, nfft_kwargs in tested_nfft_args:
        plan = NFFT(N, M, **nfft_kwargs)
        vrand_shifted_unit_double(plan.x.ravel())
        plan.precompute()
        yield check_adjoint_ndft, plan




# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import copy
import numpy as np
cimport numpy as np
from libc.stdlib cimport malloc, free
from libc cimport limits
from cnfft3 cimport *

cdef extern from *:
    int Py_AtExit(void (*callback)()) 

# Initialize module
# Numpy must be initialized. When using numpy from C or Cython you must
# _always_ do that, or you will have segfaults
np.import_array()

# initialize FFTW threads
fftw_init_threads()

# register cleanup callbacks
cdef void _cleanup():
    fftw_cleanup()
    fftw_cleanup_threads()

Py_AtExit(_cleanup)


nfft_flags_dict = {
    'PRE_PHI_HUT':PRE_PHI_HUT,
    'FG_PSI':FG_PSI,
    'PRE_LIN_PSI':PRE_LIN_PSI,
    'PRE_FG_PSI':PRE_FG_PSI,
    'PRE_PSI':PRE_PSI,
    'PRE_FULL_PSI':PRE_FULL_PSI,
    'MALLOC_X':MALLOC_X,
    'MALLOC_F_HAT':MALLOC_F_HAT,
    'MALLOC_F':MALLOC_F,
    'FFT_OUT_OF_PLACE':FFT_OUT_OF_PLACE,
    'FFTW_INIT':FFTW_INIT,
    'NFFT_SORT_NODES':NFFT_SORT_NODES,
    'NFFT_OMP_BLOCKWISE_ADJOINT':NFFT_OMP_BLOCKWISE_ADJOINT,
    'PRE_ONE_PSI':PRE_ONE_PSI,
    }

nfft_flags = copy.copy(nfft_flags_dict)

fftw_flags_dict = {
    'FFTW_ESTIMATE':FFTW_ESTIMATE,
    'FFTW_DESTROY_INPUT':FFTW_DESTROY_INPUT,
    }

fftw_flags = copy.copy(fftw_flags_dict)

nfft_supported_flags_tuple = (
    'PRE_PHI_HUT',
    'FG_PSI',
    'PRE_LIN_PSI',
    'PRE_FG_PSI',
    'PRE_PSI',
    'PRE_FULL_PSI',
    )

nfft_supported_flags = copy.copy(nfft_supported_flags_tuple)


cdef class NFFT(object):
    '''
    NFFT is a class for computing the multivariate Non-uniform Discrete
    Fourier (NDFT) transform using the NFFT library. The interface is
    designed to be somewhat pythonic, whilst preserving the workflow of the 
    original C-library. Computation of the NFFT is achieved in 3 steps : 
    instantiation, precomputation and execution.
    
    On instantiation, the geometry of the transform is provided. Optional
    computation parameters may also be defined.
    
    Precomputation initializes the internals of the transform prior to 
    execution. First the non-uniform locations must be given to the plan 
    via its :attr:`x` attribute. Computation can then be called with the 
    :meth:`precompute` method.
    
    The forward and adjoint NFFT can be eventually performed by calling the 
    :meth:`forward` and :meth:`adjoint` methods respectively. The input/output 
    of the transform can be read/written by access to the :attr:`f` and 
    :attr:`f_hat` attributes.
    '''

    # where the C-related content of the class is being initialized
    def __cinit__(self, N, M, n=None, m=12, flags=None, *args, **kwargs):

        # support only double / double complex NFFT
        # TODO: if support for multiple floating precision lands in the
        # NFFT library, adapt this section to dynamically figure the
        # real and complex dtypes
        dtype_real = np.dtype('float64')
        dtype_complex = np.dtype('complex128')

        # sanity checks on geometry parameters
        try:
            N = tuple(N)
        except TypeError:
            N = (N,)

        if n is None:
            n = [2 * Nt for Nt in N]
        
        try:
            n = tuple(n)
        except TypeError:
            n = (n,)
        
        d = len(N)
        N_total = np.prod(N)
        n_total = np.prod(n)
 
        if len(n) != d:
            raise ValueError('n should be of same length as N')       

        # check geometry is compatible with C-class internals
        int_max = <Py_ssize_t>limits.INT_MAX
        if not all([Nt > 0 for Nt in N]):
            raise ValueError('N must be strictly positive')
        if not all([Nt < int_max for Nt in N]):
            raise ValueError('N exceeds integer limit value')
        if not N_total < int_max:
            raise ValueError('product of N exceeds integer limit value')
        if not all([nt > 0 for nt in n]):
            raise ValueError('n must be strictly positive')
        if not all([nt < int_max for nt in n]):
            raise ValueError('n exceeds integer limit value')        
        if not n_total < int_max:
            raise ValueError('product of n exceeds integer limit value')
        if not M > 0:
            raise ValueError('M must be strictly positive')
        if not M < int_max:
            raise ValueError('M exceeds integer limit value')
        if not m > 0:
            raise ValueError('m must be strictly positive')
        if not m < int_max:
            raise ValueError('m exceeds integer limit value')

        # safeguard against oversampled gridsize too small for kernel size
        if not all([nt > m for nt in n]):
            raise ValueError('n must be higher than m')

        # convert tuple of litteral precomputation flags to its expected
        # C-compatible value. Each flag is a power of 2, which allows to compute
        # this value using BITOR operations.
        cdef unsigned int _nfft_flags = 0
        cdef unsigned int _fftw_flags = 0
        flags_used = ()

        # sanity checks on user specified flags if any,
        # else use default ones:
        if flags is not None:
            try:
                flags = tuple(flags)
            except:
                flags = (flags,)
            finally:
                for each_flag in flags:
                    if each_flag not in nfft_supported_flags_tuple:
                        raise ValueError('Unsupported flag: %s'%(each_flag))
                flags_used += flags
        else:
            flags_used += ('PRE_PHI_HUT', 'PRE_PSI',)

        # set specific flags, for which we don't want the user to have a say
        # on:
        # FFTW specific flags
        flags_used += ('FFTW_INIT', 'FFT_OUT_OF_PLACE', 'FFTW_ESTIMATE',
                'FFTW_DESTROY_INPUT',)

        # memory allocation flags
        flags_used += ('MALLOC_F', 'MALLOC_F_HAT', 'MALLOC_X')

        # Parallel computation flag
        flags_used += ('NFFT_SORT_NODES',)

        # Parallel computation flag, set only if multivariate transform
        if d > 1:
            flags_used += ('NFFT_OMP_BLOCKWISE_ADJOINT',)

        # Calculate the flag code for the guru interface used for
        # initialization
        for each_flag in flags_used:
            try:
                _nfft_flags |= nfft_flags_dict[each_flag]
            except KeyError:
                try:
                    _fftw_flags |= fftw_flags_dict[each_flag]
                except KeyError:
                    raise ValueError('Invalid flag: ' + '\'' +
                        each_flag + '\' is not a valid flag.')

        # initialize plan
        cdef int *p_N = <int *>malloc(sizeof(int) * d)
        if p_N == NULL:
            raise MemoryError
        for t in range(d):
            p_N[t] = N[t]

        cdef int *p_n = <int *>malloc(sizeof(int) * d)
        if p_n == NULL:
            raise MemoryError
        for t in range(d):
            p_n[t] = n[t]

        nfft_init_guru(&self._plan, d, p_N, M, p_n, m,
            _nfft_flags, _fftw_flags)
        
        free(p_N)
        free(p_n)

        # create array views
        cdef np.npy_intp *shape_f_hat
        shape_f_hat = <np.npy_intp *> malloc(d * sizeof(np.npy_intp))
        if shape_f_hat == NULL:
            raise MemoryError
        for dt in range(d):
            shape_f_hat[dt] = N[dt]

        self._f_hat = np.PyArray_SimpleNewFromData(d, shape_f_hat,
            np.NPY_COMPLEX128, <void *>(self._plan.f_hat))

        free(shape_f_hat)

        cdef np.npy_intp shape_f[1]
        shape_f[0] = M

        self._f = np.PyArray_SimpleNewFromData(1, shape_f,
            np.NPY_COMPLEX128, <void *>(self._plan.f))

        cdef np.npy_intp shape_x[2]
        shape_x[0] = M
        shape_x[1] = d

        self._x = np.PyArray_SimpleNewFromData(2, shape_x,
            np.NPY_FLOAT64, <void *>(self._plan.x))

        # initialize class members
        self._d = d
        self._M = M
        self._m = m
        self._N = N
        self._n = n
        self._dtype = dtype_complex
        self._flags = flags_used


    # here, just holds the documentation of the class constructor
    def __init__(self, N, M, n=None, m=12, flags=None, *args, **kwargs):
        '''
        :param N: multi-bandwith.
        :type N: tuple of int
        :param M: total number of samples.
        :type n: int
        :param n: oversampled multi-bandwith, default to 2 * N.
        :type n: tuple of int
        :param m: Cut-off parameter of the window function.
        :type m: int
        :param flags: list of precomputation flags, see note below.
        :type flags: tuple
        
        **Precomputation flags**

        This table lists the supported precomputation flags for the NFFT.

        +----------------------------+--------------------------------------------------+
        | Flag                       | Description                                      |
        +============================+==================================================+
        | PRE_PHI_HUT                | Precompute the roll-off correction coefficients. |
        +----------------------------+--------------------------------------------------+
        | FG_PSI                     | Convolution uses Fast Gaussian properties.       |
        +----------------------------+--------------------------------------------------+
        | PRE_LIN_PSI                | Convolution uses a precomputed look-up table.    |
        +----------------------------+--------------------------------------------------+
        | PRE_FG_PSI                 | Precompute Fast Gaussian.                        |
        +----------------------------+--------------------------------------------------+
        | PRE_PSI                    | Standard precomputation, uses M*(2m+2)*d values. |
        +----------------------------+--------------------------------------------------+
        | PRE_FULL_PSI               | Full precomputation, uses M*(2m+2)^d values.     |
        +----------------------------+--------------------------------------------------+

        Default value is ``flags = ('PRE_PHI_HUT', 'PRE_PSI')``.
        '''
        pass

    def __dealloc__(self):
        nfft_finalize(&self._plan)

    def precompute(self):
        '''Precomputes the NFFT plan internals.'''
        self._precompute()

    def trafo(self, use_dft=False):
        '''Performs the forward NFFT.

        :param use_dft: whether to use the DFT instead of the fast algorithm.
        :type use_dft: boolean
        :returns: the updated f array.
        :rtype: ndarray

        '''
        if use_dft:
            self._trafo_direct()
        else:
            self._trafo()
        return self.f
    
    def adjoint(self, use_dft=False):
        '''Performs the adjoint NFFT.

        :param use_dft: whether to use the DFT instead of the fast algorithm.
        :type use_dft: boolean
        :returns: the updated f_hat array.
        :rtype: ndarray

        '''
        if use_dft:
            self._adjoint_direct()
        else:
            self._adjoint()
        return self.f_hat

    cdef void _precompute(self):
        with nogil:
            nfft_precompute_one_psi(&self._plan)

    cdef void _trafo(self):
        with nogil:
            nfft_trafo(&self._plan)

    cdef void _trafo_direct(self):
        with nogil:
            nfft_trafo_direct(&self._plan)

    cdef void _adjoint(self):
        with nogil:
            nfft_adjoint(&self._plan)

    cdef void _adjoint_direct(self):
        with nogil:
            nfft_adjoint_direct(&self._plan)

    property f:
        
        '''The vector of non-uniform samples.'''

        def __get__(self):
            return self._f

        def __set__(self, array):
            self._f.ravel()[:] = array.ravel()

    property f_hat:

        '''The vector of Fourier coefficients.'''
        
        def __get__(self):
            return self._f_hat

        def __set__(self, array):
            self._f_hat.ravel()[:] = array.ravel()

    property x:
    
        '''The nodes in time/spatial domain.'''
        
        def __get__(self):
            return self._x

        def __set__(self, array):
            self._x.ravel()[:] = array.ravel()
   
    @property
    def d(self):
        '''The dimensionality of the NFFT.'''
        return self._d

    @property
    def m(self):
        '''The cut-off parameter of the window function.'''
        return self._m

    @property
    def M(self):
        '''The total number of samples.'''
        return self._M

    @property
    def N_total(self):
        '''The total number of Fourier coefficients.'''
        return np.prod(self.N)

    @property
    def N(self):
        '''The multi-bandwith size.'''
        return self._N

    @property
    def n(self):
        '''The oversampled multi-bandwith size.'''
        return self._n

    @property
    def dtype(self):
        '''The dtype of the NFFT.'''
        return self._dtype

    @property
    def flags(self):
        '''The precomputation flags.'''
        return self._flags




# -*- coding: utf-8 -*-
#
# Copyright (C) 2013-2014  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import numpy as np
cimport numpy as np
from libc.stdlib cimport malloc, free
from cnfft3 cimport solver_plan_complex
from .nfft cimport NFFT

cdef object solver_flags_dict

cdef class Solver(object):
    cdef solver_plan_complex _solver_plan
    cdef NFFT _nfft_plan
    cdef object _w
    cdef object _w_hat
    cdef object _y
    cdef object _f_hat_iter
    cdef object _r_iter
    cdef object _z_hat_iter
    cdef object _p_hat_iter
    cdef object _v_iter
    cdef object _dtype
    cdef object _flags
    cdef void _before_loop(self)
    cdef void _loop_one_step(self)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import subprocess


# Define global path variables
setup_dir = dir = os.path.dirname(os.path.abspath(__file__))
package_name = 'pynfft'
package_dir = os.path.join(setup_dir, package_name)


# Define utility functions to build the extensions
def get_common_extension_args():
    import numpy
    common_extension_args = dict(
        libraries=['nfft3_threads', 'nfft3', 'fftw3_threads', 'fftw3', 'm'],
        library_dirs=[],
        include_dirs=[numpy.get_include()],
        extra_compile_args='-O3 -fomit-frame-pointer -malign-double '
        '-fstrict-aliasing -ffast-math'.split(),
        )
    return common_extension_args

def get_extensions():
    from distutils.extension import Extension
    ext_modules = []
    common_extension_args = get_common_extension_args()
    ext_modules.append(Extension(
            name=package_name+'.nfft',
            sources=[os.path.join(package_dir, 'nfft.c')],
            **common_extension_args
            )
        )
    ext_modules.append(Extension(
            name=package_name+'.solver',
            sources=[os.path.join(package_dir, 'solver.c')],
            **common_extension_args
            )
        )
    ext_modules.append(Extension(
            name=package_name+'.util',
            sources=[os.path.join(package_dir, 'util.c')],
            **common_extension_args
            )
        )
    return ext_modules

def get_cython_extensions():
    from distutils.extension import Extension
    from Cython.Build import cythonize
    ext_modules = []
    common_extension_args = get_common_extension_args()
    ext_modules.append(Extension(
            name=package_name+'.nfft',
            sources=[os.path.join(package_dir, 'nfft.pyx')],
            **common_extension_args
            )
        )
    ext_modules.append(Extension(
            name=package_name+'.solver',
            sources=[os.path.join(package_dir, 'solver.pyx')],
            **common_extension_args
            )
        )
    ext_modules.append(Extension(
            name=package_name+'.util',
            sources=[os.path.join(package_dir, 'util.pyx')],
            **common_extension_args
            )
        )
    return cythonize(ext_modules)


# BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
# update it when the contents of directories change.
if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')


# Define custom clean command
from distutils.core import Command
class CleanCommand(Command):
    """Custom distutils command to clean the .so and .pyc files."""

    user_options = [("all", "a", "")]

    def initialize_options(self):
        self.all = True
        self._clean_me = []
        self._clean_trees = []
        self._clean_exclude = []
        # clean Cython generated files and cache
        for root, dirs, files in os.walk(package_dir):
            for f in files:
                if f in self._clean_exclude:
                    continue
                if os.path.splitext(f)[-1] in ('.pyc', '.so', '.o',
                                               '.pyo',
                                               '.pyd', '.c', '.orig'):
                    self._clean_me.append(os.path.join(root, f))
            for d in dirs:
                if d == '__pycache__':
                    self._clean_trees.append(os.path.join(root, d))
        # clean build and sdist directories in root
        for d in ('build', 'dist'):
            if os.path.exists(d):
                self._clean_trees.append(d)

    def finalize_options(self):
        pass

    def run(self):
        for clean_me in self._clean_me:
            try:
                os.unlink(clean_me)
            except Exception:
                pass
        for clean_tree in self._clean_trees:
            try:
                shutil.rmtree(clean_tree)
            except Exception:
                pass

cmdclass = {'clean': CleanCommand}


LONG_DESCRIPTION = '''"The NFFT is a C subroutine library for computing the
nonequispaced discrete Fourier transform (NDFT) in one or more dimensions, of
arbitrary input size, and of complex data."

The NFFT library is licensed under GPLv2 and available at:
    http://www-user.tu-chemnitz.de/~potts/nfft/index.php

This wrapper provides a somewhat Pythonic access to some of the core NFFT
library functionalities and is largely inspired from the pyFFTW project
developped by Henry Gomersall (http://hgomersall.github.io/pyFFTW/).

This project is still work in progress and is still considered beta quality. In
particular, the API is not yet frozen and is likely to change as the
development continues. Please consult the documentation and changelog for more
information.'''

CLASSIFIERS = [
    'Programming Language :: Cython',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Development Status :: 4 - Beta',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Operating System :: POSIX :: Linux',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering',
    'Topic :: Scientific/Engineering :: Mathematics',
    'Topic :: Multimedia :: Sound/Audio :: Analysis',
]

MAJOR = 1
MINOR = 3
MICRO = 2
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)

# borrowed from scipy
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout = subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION

# borrowed from scipy
def get_version_info():
    FULLVERSION = VERSION
    if os.path.exists('.git'):
        GIT_REVISION = git_version()
    elif os.path.exists('pynfft/version.py'):
        # must be a source distribution, use existing version file
        # load it as a separate module in order not to load __init__.py
        import imp
        version = imp.load_source('pynfft.version', 'pynfft/version.py')
        GIT_REVISION = version.git_revision
    else:
        GIT_REVISION = "Unknown"

    if not ISRELEASED:
        FULLVERSION += '.dev-' + GIT_REVISION[:7]

    return FULLVERSION, GIT_REVISION

# borrowed from scipy
def write_version_py(filename='pynfft/version.py'):
    cnt = """
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s

if not release:
    version = full_version
"""
    FULLVERSION, GIT_REVISION = get_version_info()

    f = open(filename, 'w')
    try:
        f.write(cnt % {'version': VERSION,
                       'full_version' : FULLVERSION,
                       'git_revision' : GIT_REVISION,
                       'isrelease': str(ISRELEASED)})
    finally:
        f.close()


def setup_package():
    # Use setuptools if available
    try:
        from setuptools import setup
    except ImportError:
        from distutils.core import setup
    
    # Get current version
    FULLVERSION, GIT_REVISION = get_version_info()
    
    # Refresh version file
    write_version_py()
    
    # Figure out whether to add ``*_requires = ['numpy']``.
    build_requires = []
    try:
        import numpy
    except:
        build_requires = ['numpy>=1.6',]
        
    # Common setup args
    setup_args = dict(
        name = 'pyNFFT',
        version = FULLVERSION,
        author = 'Ghislain Vaillant',
        author_email = 'ghisvail@gmail.com',
        description = 'A pythonic wrapper around NFFT',
        long_description = LONG_DESCRIPTION,
        url = 'https://github.com/ghisvail/pyNFFT.git',
        cmdclass = cmdclass,
        classifiers = CLASSIFIERS,
        platforms=['Linux', 'Unix'],
        test_suite='nose.collector',
        setup_requires = build_requires,
        install_requires = build_requires,
        )
        
    if len(sys.argv) >= 2 and ('--help' in sys.argv[1:] or
            sys.argv[1] in ('--help-commands', 'egg_info', '--version',
                            'clean')):
        # For these actions, NumPy is not required.
        pass
    else:
        try:
            from Cython.Distutils import build_ext
            have_cython = True
        except:
            have_cython = False
        if have_cython:
            extensions = get_cython_extensions()
        else:
            extensions = get_extensions()
        setup_args['packages'] = ['pynfft', 'pynfft.tests']
        setup_args['ext_modules'] = extensions
        
    setup(**setup_args)


if __name__ == '__main__':
    setup_package()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

cdef extern from "nfft3.h":

    # precomputation flags for the NFFT component
    ctypedef enum:
        PRE_PHI_HUT      #(1U<< 0)
        FG_PSI           #(1U<< 1)
        PRE_LIN_PSI      #(1U<< 2)
        PRE_FG_PSI       #(1U<< 3)
        PRE_PSI          #(1U<< 4)
        PRE_FULL_PSI     #(1U<< 5)
        MALLOC_X         #(1U<< 6)
        MALLOC_F_HAT     #(1U<< 7)
        MALLOC_F         #(1U<< 8)
        FFT_OUT_OF_PLACE #(1U<< 9)
        FFTW_INIT        #(1U<< 10)
        NFFT_SORT_NODES  #(1U<< 11)
        NFFT_OMP_BLOCKWISE_ADJOINT #(1U<<12)
        PRE_ONE_PSI #(PRE_LIN_PSI| PRE_FG_PSI| PRE_PSI| PRE_FULL_PSI)

    # precomputation flags for the FFTW component
    ctypedef enum:
        FFTW_ESTIMATE
        FFTW_DESTROY_INPUT

    # double precision complex type
    ctypedef double fftw_complex[2]

    # double precision NFFT plan
    ctypedef struct nfft_plan:
        fftw_complex *f_hat
            # Vector of Fourier coefficients, size is N_total float_types.
        fftw_complex *f
            # Vector of samples, size is M_total float types.
        double *x
            # Nodes in time/spatial domain, size is $dM$ doubles.

    void nfft_trafo_direct (nfft_plan *ths) nogil
        # Computes a NDFT.

    void nfft_adjoint_direct (nfft_plan *ths) nogil
        # Computes an adjoint NDFT.

    void nfft_trafo (nfft_plan *ths) nogil
        # Computes a NFFT, see the definition.

    void nfft_adjoint (nfft_plan *ths) nogil
        # Computes an adjoint NFFT, see the definition.

    void nfft_init_guru (nfft_plan *ths, int d, int *N, int M, int *n, int m,
                         unsigned nfft_flags, unsigned fftw_flags)
        # Initialisation of a transform plan, guru.

    void nfft_precompute_one_psi (nfft_plan *ths) nogil
        # Precomputation for a transform plan.

    void nfft_finalize (nfft_plan *ths)
        # Destroys a transform plan.


    ctypedef enum:
        # precomputation flags for solver
        LANDWEBER             #(1U<< 0)
        STEEPEST_DESCENT      #(1U<< 1)
        CGNR                  #(1U<< 2)
        CGNE                  #(1U<< 3)
        NORMS_FOR_LANDWEBER   #(1U<< 4)
        PRECOMPUTE_WEIGHT     #(1U<< 5)
        PRECOMPUTE_DAMP       #(1U<< 6)

    # stripped down alias of a NFFT plan used by solver
    ctypedef struct nfft_mv_plan_complex:
        pass

    # complex solver plan
    ctypedef struct solver_plan_complex:
        nfft_mv_plan_complex *mv
            # matrix vector multiplication.
        unsigned flags
            # iteration type
        double *w
            # weighting factors
        double *w_hat
            # damping factors
        fftw_complex *y
            # right hand side, samples
        fftw_complex *f_hat_iter
            # iterative solution
        fftw_complex *r_iter
            # iterated residual vector
        fftw_complex *z_hat_iter
            # residual of normal equation of \ first kind
        fftw_complex *p_hat_iter
            # search direction.
        fftw_complex *v_iter
            # residual vector update
        double alpha_iter
            #  step size for search direction
        double beta_iter
            #  step size for search correction
        double dot_r_iter
            #  weighted dotproduct of r_iter
        double dot_r_iter_old
            #  previous dot_r_iter
        double dot_z_hat_iter
            #  weighted dotproduct of \ z_hat_iter
        double dot_z_hat_iter_old
            #  previous dot_z_hat_iter
        double dot_p_hat_iter
            #  weighted dotproduct of \ p_hat_iter
        double dot_v_iter
            # weighted dotproduct of v_iter

    void solver_init_advanced_complex(solver_plan_complex *ths,
                                      nfft_mv_plan_complex *mv,
                                      unsigned flags)
        # Advanced initialisation.

    void solver_before_loop_complex(solver_plan_complex *ths) nogil
        # Setting up residuals before the actual iteration.

    void solver_loop_one_step_complex(solver_plan_complex *ths) nogil
        # Doing one step in the iteration.

    void solver_finalize_complex(solver_plan_complex *ths)
        # Destroys the plan for the inverse transform.


cdef extern from "fftw3.h":
    
    void fftw_cleanup()
        # cleanup routines
        
    void fftw_init_threads()
        # threading routines

    void fftw_cleanup_threads()
        # cleanup routines



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013-2014  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import copy
import numpy as np
cimport numpy as np
from libc.stdlib cimport malloc, free
from cnfft3 cimport *
from .nfft cimport NFFT
from .nfft import NFFT

# Initialize module
# Numpy must be initialized. When using numpy from C or Cython you must
# _always_ do that, or you will have segfaults
np.import_array()

solver_flags_dict = {
    'LANDWEBER':LANDWEBER,
    'STEEPEST_DESCENT':STEEPEST_DESCENT,
    'CGNR':CGNR,
    'CGNE':CGNE,
    'NORMS_FOR_LANDWEBER':NORMS_FOR_LANDWEBER,
    'PRECOMPUTE_WEIGHT':PRECOMPUTE_WEIGHT,
    'PRECOMPUTE_DAMP':PRECOMPUTE_DAMP,
    }

solver_flags = copy.copy(solver_flags_dict)


cdef class Solver(object):
    '''
    Solver is a class for computing the inverse NFFT iteratively.

    The solver's instantiation requires an initialized NFFT object used 
    internally for the multiple forward and adjoint NFFT performed. The class 
    uses conjugate-gradient as the default solver but alternative solvers may 
    be specified at construct-time.

    The solver must be first initialized by calling the :meth:`before_loop` 
    method.

    The solver's implementation let you carry one iteration at a time with a 
    call to :meth:`loop_one_step`. It is left to the user to chose whichever
    stopping condition to apply.
    
    The class exposes the internals of the solver through its respective 
    properties. For instance, the residuals for the current iteration can be
    accessed via the :attr:r_iter attribute.
    '''

    def __cinit__(self, NFFT nfft_plan, flags=None):

        # support only double / double complex NFFT
        # TODO: if support for multiple floating precision lands in the
        # NFFT library, adapt this section to dynamically figure the
        # real and complex dtypes
        dtype_real = np.dtype('float64')
        dtype_complex = np.dtype('complex128')

        # convert tuple of litteral precomputation flags to its expected
        # C-compatible value. Each flag is a power of 2, which allows to compute
        # this value using BITOR operations.
        cdef unsigned int _flags = 0
        flags_used = ('PRECOMPUTE_WEIGHT', 'PRECOMPUTE_DAMP')

        # sanity checks on user specified flags if any,
        # else use default ones:
        if flags is not None:
            try:
                flags = tuple(flags)
            except:
                flags = (flags,)
            finally:
                flags_used += flags
        else:
            flags_used += ('CGNR',)

        for each_flag in flags_used:
            try:
                _flags |= solver_flags_dict[each_flag]
            except KeyError:
                raise ValueError('Invalid flag: ' + '\'' +
                        each_flag + '\' is not a valid flag.')

        # initialize plan
        try:
            solver_init_advanced_complex(&self._solver_plan,
                <nfft_mv_plan_complex*>&(nfft_plan._plan), _flags)
        except:
            raise MemoryError

        self._nfft_plan = nfft_plan
        d = nfft_plan.d
        M = nfft_plan.M
        N = nfft_plan.N

        cdef np.npy_intp shape_M[1]
        shape_M[0] = M

        cdef np.npy_intp *shape_N
        try:
            shape_N = <np.npy_intp*>malloc(d*sizeof(np.npy_intp))
        except:
            raise MemoryError
        for dt in range(d):
            shape_N[dt] = N[dt]

        self._w = np.PyArray_SimpleNewFromData(1, shape_M,
            np.NPY_FLOAT64, <void *>(self._solver_plan.w))
        self._w.ravel()[:] = 1  # make sure weights are initialized

        self._w_hat = np.PyArray_SimpleNewFromData(d, shape_N,
            np.NPY_FLOAT64, <void *>(self._solver_plan.w_hat))
        self._w_hat.ravel()[:] = 1  # make sure weights are initialized

        self._y = np.PyArray_SimpleNewFromData(1, shape_M,
            np.NPY_COMPLEX128, <void *>(self._solver_plan.y))

        self._f_hat_iter = np.PyArray_SimpleNewFromData(d, shape_N,
            np.NPY_COMPLEX128, <void *>(self._solver_plan.f_hat_iter))
        self._f_hat_iter.ravel()[:] = 0  # default initial guess

        self._r_iter = np.PyArray_SimpleNewFromData(1, shape_M,
            np.NPY_COMPLEX128, <void *>(self._solver_plan.r_iter))

        self._z_hat_iter = np.PyArray_SimpleNewFromData(d, shape_N,
            np.NPY_COMPLEX128, <void *>(self._solver_plan.z_hat_iter))

        self._p_hat_iter = np.PyArray_SimpleNewFromData(d, shape_N,
            np.NPY_COMPLEX128, <void *>(self._solver_plan.p_hat_iter))

        self._v_iter = np.PyArray_SimpleNewFromData(1, shape_M,
            np.NPY_COMPLEX128, <void *>(self._solver_plan.v_iter))

        free(shape_N)

        self._dtype = dtype_complex
        self._flags = flags_used


    def __init__(self, nfft_plan, flags=None):
        '''
        :param plan: instance of NFFT.
        :type plan: :class:`NFFT`
        :param flags: list of instantiation flags, see below.
        :type flags: tuple

        **Instantiation flags**

        +---------------------+-----------------------------------------------------------------------------+
        | Flag                | Description                                                                 |
        +=====================+=============================================================================+
        | LANDWEBER           | Use Landweber (Richardson) iteration.                                       |
        +---------------------+-----------------------------------------------------------------------------+
        | STEEPEST_DESCENT    | Use steepest descent iteration.                                             |
        +---------------------+-----------------------------------------------------------------------------+
        | CGNR                | Use conjugate gradient (normal equation of the 1st kind).                   |
        +---------------------+-----------------------------------------------------------------------------+
        | CGNE                | Use conjugate gradient (normal equation of the 2nd kind).                   |
        +---------------------+-----------------------------------------------------------------------------+
        | NORMS_FOR_LANDWEBER | Use Landweber iteration to compute the residual norm.                       |
        +---------------------+-----------------------------------------------------------------------------+
        | PRECOMPUTE_WEIGHT   | Weight the samples, e.g. to cope with varying sampling density.             |
        +---------------------+-----------------------------------------------------------------------------+
        | PRECOMPUTE_DAMP     | Weight the Fourier coefficients, e.g. to favour fast decaying coefficients. |
        +---------------------+-----------------------------------------------------------------------------+

        Default value is ``flags = ('CGNR',)``.
        '''
        pass

    def __dealloc__(self):
        solver_finalize_complex(&self._solver_plan)

    def before_loop(self):
        '''Initialize the solver internals.'''
        self._before_loop()

    def loop_one_step(self):
        '''Perform one iteration of the solver.'''
        self._loop_one_step()

    cdef void _before_loop(self):
        with nogil:
            solver_before_loop_complex(&self._solver_plan)

    cdef void _loop_one_step(self):
        with nogil:
            solver_loop_one_step_complex(&self._solver_plan)

    property w:
        
        '''Weighting factors.'''
        
        def __get__(self):
            return self._w
        
        def __set__(self, array):
            self._w.ravel()[:] = array.ravel()

    property w_hat:
        
        '''Damping factors.'''
        
        def __get__(self):
            return self._w_hat
        
        def __set__(self, array):
            self._w_hat.ravel()[:] = array.ravel()

    property y:
        
        '''Right hand side, samples.'''
        
        def __get__(self):
            return self._y
        
        def __set__(self, array):
            self._y.ravel()[:] = array.ravel()

    property f_hat_iter:

        '''Iterative solution.'''

        def __get__(self):
            return self._f_hat_iter

        def __set__(self, array):
            self._f_hat_iter.ravel()[:] = array.ravel()

    @property
    def r_iter(self):
        '''Residual vector.'''
        return self._r_iter

    @property
    def z_hat_iter(self):
        '''Residual of normal equation of the first kind.'''
        return self._z_hat_iter

    @property
    def p_hat_iter(self):
        '''Search direction.'''
        return self._p_hat_iter

    @property
    def v_iter(self):
        '''Residual vector update.'''
        return self._v_iter

    @property
    def alpha_iter(self):
        '''Step size for search direction.'''
        return self._solver_plan.alpha_iter

    @property
    def beta_iter(self):
        '''Step size for search direction.'''
        return self._solver_plan.beta_iter

    @property
    def dot_r_iter(self):
        '''Weighted dotproduct of r_iter.'''
        return self._solver_plan.dot_r_iter

    @property
    def dot_r_iter_old(self):
        '''Previous dot_r_iter.'''
        return self._solver_plan.dot_r_iter_old

    @property
    def dot_z_hat_iter(self):
        '''Weighted dotproduct of z_hat_iter.'''
        return self._solver_plan.dot_z_hat_iter

    @property
    def dot_z_hat_iter_old(self):
        '''Previous dot_z_hat_iter.'''
        return self._solver_plan.dot_z_hat_iter_old

    @property
    def dot_p_hat_iter(self):
        '''Weighted dotproduct of p_hat_iter.'''
        return self._solver_plan.dot_p_hat_iter

    @property
    def dot_v_iter(self):
        '''Weighted dotproduct of v_iter.'''
        return self._solver_plan.dot_v_iter

    @property
    def dtype(self):
        '''The dtype of the solver.'''
        return self._dtype

    @property
    def flags(self):
        '''The precomputation flags.'''
        return self._flags



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Ghislain Vaillant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import numpy as np
cimport numpy as np
from cnfft3util cimport *


# Numpy must be initialized. When using numpy from C or Cython you must
# _always_ do that, or you will have segfaults
np.import_array()

def vrand_unit_complex (object[np.complex128_t, mode='c'] x not None):
    '''
    Utilitary function for initializing a vector of knots to random
    values within the range [-0.5, 0.5).

    Used for testing :attr:`pynfft.NFFT.x`.

    :param x: pre-allocated array
    :type x: ndarray <complex128>
    '''
    nfft_vrand_unit_complex(<fftw_complex *>&x[0], x.size)

def vrand_shifted_unit_double (object[np.float64_t, mode='c'] x not None):
    '''
    Utilitary function for initializing a vector of data to random
    complex values within the range [0, 1).

    Used for testing :attr:`pynfft.NFFT.f` and
    :attr:`pynfft.NFFT.f_hat`.

    :param x: pre-allocated array
    :type x: ndarray <float64>
    '''
    nfft_vrand_shifted_unit_double(<double *>&x[0], x.size)

def voronoi_weights_1d (object[np.float64_t, mode='c'] w not None,
                        object[np.float64_t, mode='c'] x not None):
    '''
    Utilitary function for computing density compensation weights from 1D knots.

    :param w: pre-allocated array
    :type w: ndarray <float64>
    :param x: ordered 1D knots
    :type x: ndarray <float64>
    '''
    if x.size != w.size:
        raise ValueError('Incompatible size between weights and nodes \
                         (%d, %d)'%(w.size, x.size))
    nfft_voronoi_weights_1d(<double *>&w[0], <double *>&x[0], w.size)

def voronoi_weights_S2 (object[np.float64_t, mode='c'] w not None,
                        object[np.float64_t, mode='c'] xi not None):
    '''
    Utilitary function for computing density compensation weights from knots
    located on the surface of a sphere.

    Useful for reconstruction of 3D radial data.

    :param w: pre-allocated array
    :type w: ndarray <float64>
    :param xi: angular locations (2D) on the unit sphere
    :type xi: ndarray <float64>
    '''
    if xi.size != 2 * w.size:
        raise ValueError('Incompatible size between weights and nodes \
                         (%d, %d)'%(w.size, xi.size))
    nfft_voronoi_weights_S2(<double *>&w[0], <double *>&xi[0], w.size)
